/*
 * Copyright (c) 2004 Thomas Nystrm and Stacken Computer Club
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

typedef struct {
	SOCKET sock;
	TEventSemaphore *event;
	} SocketChannel;

class TSaslError : public xmsg
{
      public:
	TSaslError(const char *s);
	TSaslError(const char *s, int errcode);
	TSaslError(const char *s, const char *s1);

      private:
	string MakeString(const char *s, const char *s1);
	char pbuf[32];

};

class TSasl
{
      public:
	TSasl(const char *_host, const char *_user);
	~TSasl();
	void ProtocolInit(SocketChannel *_sock, TEventSemaphore *_ExitSemaphore);
	void GetLine(char **line);
	void SendCmd(const char *fmt, ...);
	bool WaitForEvent(SocketChannel &sock, ulong timeOut = INFINITE);
	int Getc();
	size_t Read(unsigned char *buf, size_t max);
	size_t Write(const unsigned char *buf, size_t size);
	char *Authenticate();

      private:
	enum {SASL_GREET, SASL_CAPA, SASL_SELECT_AUTH, 
	      SASL_AUTH, SASL_AUTH_OK, SASL_VERIFY_CAPA, 
	      SASL_DONE, SASL_FAILED } sasl_state;
	int flags;	      
	struct tsasl_peer *client;
	bool EncryptOn;
	TEventSemaphore *ExitSemaphore;
	SocketChannel *sock;
	struct tsasl_prot *ctx;
	const char *Host;
	const char *User;
	char rcv_buf[32768];
	int rcv_valid;
	int rcv_pek;

	bool FillBuffer(bool WaitFlag);

};



