/*
 * Copyright (c) 2000 Thomas Nystrm and Stacken Computer Club
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "common.h"
#pragma hdrstop

#include "KClient.h"
#include "kclient.rh"

#include <krb.h>
#include <setup.rh>

#include "passdia.h"

//****************************************************************

class KCRealmSel : public TDialog
{
      public:
	KCRealmSel(CrossRealm *parent, char **Realms);
	~KCRealmSel();

	virtual void SetupWindow();
	virtual void CloseWindow(int code);
	void CmCommon(WPARAM wParam);
	void EvClose();
	Kerr Select();
	CrossRealm *Parent;
	HWND OldForeground;
	Kerr err;
		
	struct {
		TComboBoxData realms;
		} TrfBuf;
DECLARE_RESPONSE_TABLE(KCRealmSel);
};

//****************************************************************

inline static void
dcpy(char *d, const char *s)
{
	if (s)
		strlcpy(d, s, KCP_MAXSTR);
	else
		d[0] = 0;
}

//****************************************************************

KCPBuf::KCPBuf(const char *u, const char *i, const char *r, const char *p)
{
	dcpy(user, u);
	dcpy(instance, i);
	dcpy(realm, r);
	dcpy(passwd, p);

	if (realm[0] == 0)
		krb_get_lrealm(realm, 1);
}

KCPBuf::~KCPBuf()
{
	ZeroMemory(passwd, KCP_MAXSTR);
}

//****************************************************************

DEFINE_RESPONSE_TABLE1(KCRealmSel, TDialog)
	EV_WM_CLOSE,
	EV_COMMAND_AND_ID(IDC_NEW, CmCommon),
END_RESPONSE_TABLE;

KCRealmSel::KCRealmSel(CrossRealm *parent, char **Realms)
	: TDialog(parent->Parent, IDD_KCREALM, ThisModule)
{
	register int i;
	bool first;
	
	OldForeground = 0;
	Parent = parent;
	
	new TComboBox(this, IDC_REALMS);
	new TButton(this, IDC_NEW);
	SetTransferBuffer(&TrfBuf);
	
	first = true;
	for (i = 0; Realms[i]; i++) {
		TrfBuf.realms.AddString(Realms[i], first);
		first = false;
		}
}

KCRealmSel::~KCRealmSel()
{
}

void
KCRealmSel::SetupWindow()
{
	TDialog::SetupWindow();

	if (!Parent->Parent) {
		OldForeground = GetForegroundWindow();
		SetForegroundWindow(*this);
		}
}

void
KCRealmSel::CmCommon(WPARAM wParam)
{
	CloseWindow((int)wParam);
}

void
KCRealmSel::EvClose()
{
	if (OldForeground)
		SetForegroundWindow(OldForeground);
	
	TDialog::EvClose();
}

void
KCRealmSel::CloseWindow(int code)
{
	if (code == IDOK) {
		TransferData(tdGetData);
		err = Parent->TryCrossAuth(TrfBuf.realms.GetSelection().c_str());
		if (err != KSUCCESS) {
			MessageBox(Parent->ErrorText(err, 0),
				errorHeader, MB_OK|MB_ICONEXCLAMATION);
			return;
			}
		}
	else
		err = KFAILURE;

	if (OldForeground)
		SetForegroundWindow(OldForeground);

	TDialog::CloseWindow(code);
}

Kerr
KCRealmSel::Select()
{
	if (Execute() == IDCANCEL)
		return AD_NOTGT;
	return err;
}

//****************************************************************
 
KCPassword::KCPassword(TWindow *parent, KCPBuf *kcp, bool startLoginInPassw)
	: TDialog(parent, IDD_KCPSW, ThisModule)
{
	Kcp = kcp;
	StartLoginInPassw = startLoginInPassw;

	new TEdit(this, IDC_KCPUSER, KCP_MAXSTR);
	new TEdit(this, IDC_KCPINSTANCE, KCP_MAXSTR);
	new TEdit(this, IDC_KCPREALM, KCP_MAXSTR);
	new TEdit(this, IDC_KCPPSW, KCP_MAXSTR);

	OldForeground = 0;
	SetTransferBuffer((kcpb *)Kcp);
}

KCPassword::~KCPassword()
{
}

void
KCPassword::CloseWindow(int code)
{
	if (code == IDOK)
		TransferData(tdGetData);

	if (OldForeground)
		SetForegroundWindow(OldForeground);

	TDialog::CloseWindow(code);
}

void
KCPassword::SetupWindow()
{
	TDialog::SetupWindow();
	
	if (!Parent) {
		OldForeground = GetForegroundWindow();
		SetForegroundWindow(*this);
		}
}

bool
KCPassword::EvInitDialog(THandle hFocus)
{
	TEdit *e;

	TDialog::EvInitDialog(hFocus);
	
	if (!StartLoginInPassw)
		return true;

	if (Kcp->user[0] && Kcp->realm[0]) {
		// We decides who shall have focus at startup
		e = dynamic_cast<TEdit *>(ChildWithId(IDC_KCPPSW));
		if (e) {
			SetControlFocus(*e);
			return false;
			}
		}

	return true;
}

//****************************************************************

CrossRealm::CrossRealm(TWindow *parent, const char *user,
	const char *inst, char *realm, size_t realm_siz,
	int crossmode, SetupParam *setupData)
{
	Parent = parent;
	Realm = realm;
	Realm_siz = realm_siz;
	CrossMode = crossmode;
	User = user;
	Inst = inst;
	SetupData = setupData;
}

Kerr
CrossRealm::Execute()
{
	Kerr err;
	
	if (Realm[0] == 0)
		CrossMode = IDS_CMNEVER;

	if ((CrossMode != IDS_CMNEVER) && (CrossMode != IDS_CMSELECT))
		CrossMode = IDS_CMAUTO;

	err = KFAILURE;	

	if ((CrossMode == IDS_CMSELECT) || (CrossMode == IDS_CMAUTO)) {
		err = CrossAuth(CrossMode);
		if (err == AD_NOTGT)
			return KFAILURE;
		}

	if (err != KSUCCESS) {
		KCPBuf b(User, Inst, Realm, 0);

		if (KCPassword(Parent, &b,
			SetupData->StartLoginInPassw).Execute() != IDOK)
			return KFAILURE;

		err = KSUCCESS;
		if (!LoggedIn(b.realm))
			err = Login(b);
		
		if (err == KDC_PR_UNKNOWN) {
			char alt_realm[REALM_SZ];

			if (GetAltRealm(b.realm, alt_realm) == KSUCCESS) {
				if (!LoggedIn(alt_realm)) {
					err = Login(b, alt_realm);
					if (err == KSUCCESS)
						strlcpy(b.realm,
							alt_realm,
							sizeof(b.realm));
					}
				}
			}
		if (err != KSUCCESS)
			return err;

		if (!LoggedIn(Realm))
			if ((err = CrossAuth(IDS_CMAUTO)) != KSUCCESS)
				return err;

		strlcpy(Realm, b.realm, Realm_siz);
		}
	return err;
}


Kerr
CrossRealm::CrossAuth(int _CrossMode)
{
	register int i;
	char **Realms;

	Kerr err = KFAILURE;

	if ((Realms = GetRealms()) != 0) {
		if (_CrossMode == IDS_CMSELECT) {
			err = KCRealmSel(this, Realms).Select();
			}
		else {
			for (i = 0; Realms[i]; i++) {
				err = TryCrossAuth(Realms[i]);
				if (err == KSUCCESS)
					break;
				}
			}

		for (i = 0; Realms[i]; i++)
			delete[] Realms[i];
		delete[] Realms;
		}
	return err;
}

