/* -*- C++ -*- */

/*
 * Adapted for Windows 95/98/NT and Borland C++ 5.0
 * by Thomas Nystrm, Stacken Computer Club
 * 1998-12-30 20:47:46
 */	 

/*
 * Copyright (c) 1995 - 2004 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
//
//
// AuthOption
//
// Authenticates user with Kerberos 4
//

#ifndef __AUTHOPTION_H__
#define __AUTHOPTION_H__

#include "toptions.h"

#define AUTHENTICATION	37
#define ENCRYPT_OPT	38
	
//       Authentication types
#define KERBEROS_V4	1
#define KERBEROS_V5	2
#define SPX		3
#define RSA		6
#define LOKI		10
	
//       Modifiers
#define AUTH_WHO_MASK		0x01
#define AUTH_CLIENT_TO_SERVER	0x00
#define AUTH_SERVER_TO_CLIENT	0x01
#define AUTH_HOW_MASK		0x02
#define AUTH_HOW_ONE_WAY	0x00
#define AUTH_HOW_MUTUAL		0x02
	
// Kerberos 4 suboption commands
#define KRB4_AUTH		0
#define KRB4_REJECT		1
#define KRB4_ACCEPT		2
#define KRB4_CHALLENGE		3
#define KRB4_RESPONSE		4

#define	KRB5_AUTH		0	/* Authentication data follows */
#define	KRB5_REJECT		1	/* Rejected (reason might follow) */
#define	KRB5_ACCEPT		2	/* Accepted */
#define	KRB5_RESPONSE		3	/* Response for mutual auth. */

#define KRB5_FORWARD     	4       /* Forwarded credentials follow */
#define KRB5_FORWARD_ACCEPT    	5       /* Forwarded credentials accepted */
#define KRB5_FORWARD_REJECT    	6       /* Forwarded credentials rejected */


class KClient;
class Kerberos5Context;

class AuthOption : public Option
{
      public:
	AuthOption(KTelnetConnection *c, ConnectionData _cd);
	~AuthOption ();
	void NegotiateOption (char);
	bool InEffect();

      private:
	KClient *pKClient;
	ConnectionData cd;


	char challenge[8];
	void NegotiateSend();
	void NegotiateRetry();
	void NegotiateReply();
	void NegotiateEncryption();

	void Retry4(const char *error);
	void Retry5(const char *error);

	void SendNoKerb4();
	bool KerbInit();
	char noteBuf[1024];

	void NegotiateReplyKrb4(char mod);
	void NegotiateSendKrb4(char mod);
	void SendChallengeKrb4();

	void NegotiateReplyKrb5(char mod);
	void NegotiateSendKrb5(char mod);
	void SendChallengeKrb5();

	void Kerberos5Forward(char mod);

	int Data(char mod, int type, void *d, int c);
	void SendUserId();

	enum {enAuthUnk, enAuthKrb4, enAuthKrb5} AuthMode;

#define MAX_AUTH_TYPES 32
	struct {
		char type;
		char mod;
		} AuthPairs[MAX_AUTH_TYPES];
	int AuthCount, AuthIdx;
};

#endif /* __AUTHOPTION_H__ */
