/*
 * Copyright (c) 2003 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden). 
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 */

#ifndef TSASL_H
#define TSASL_H 1

#ifdef __BORLANDC__
#pragma warn -stu	// Undefined structure 'structure'
#endif

struct tsasl_peer;
struct tsasl_buffer;
struct tsasl_prot;

#ifndef TSASL_LIB_FUNCTION
#if defined(__BORLANDC__)
#define TSASL_LIB_FUNCTION _stdcall
#elif defined(_MSC_VER)
#define TSASL_LIB_FUNCTION /* not-ready-definition-yet2 */
#else
#define TSASL_LIB_FUNCTION
#endif
#endif

typedef int TSASL_LIB_FUNCTION (*tsasl_auth_db_func)(const char *, const char *, 
				  const char *, char *, size_t, void *);

#include <tsasl-protos.h>

struct tsasl_buffer {
	void	*tb_data;
	int	 tb_size;
};

#define TSASL_FLAGS_INITIATOR		0x01	/* client */
#define TSASL_FLAGS_BASE64_AUTH		0x02
#define TSASL_FLAGS_BASE64_WRAP		0x04
#define TSASL_FLAGS_CONFIDENTIALITY	0x08	/* encrypt */
#define TSASL_FLAGS_INTEGRITY		0x10	/* checksum */
#define TSASL_FLAGS_INTERACTIVE		0x20	/* client only */
#define TSASL_FLAGS_ANONYMOUS		0x40

enum	{ TSASL_DONE = 0, 
	  TSASL_CONTINUE = 100001,
	  TSASL_ERROR = 100002
};

#define var_set(_name)							\
int TSASL_LIB_FUNCTION tsasl_##_name##_set(struct tsasl_peer *, const char *);		\
int TSASL_LIB_FUNCTION tsasl_##_name##_get(struct tsasl_peer *, char **);		\
void TSASL_LIB_FUNCTION tsasl_##_name##_free(char * _name)

#ifdef __cplusplus
extern "C" {
#endif

var_set(servicename);
var_set(realm);
var_set(authorization_id);
var_set(authentication_id);

#ifdef __cplusplus
}
#endif

#undef var_set

/* protected (sasl) stream */

struct tsasl_prot;

enum { TSASL_PROT_EOF = -1 };

enum { TSASL_PROT_FLUSH = 1 };

#endif /* TSASL_H */
