#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "roken.h"

#include <stdio.h>      /* EOF definition */

#define KBD_TEXTMODE (ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_PROCESSED_INPUT)

/* The following two variables are located in kbhit.c. */

extern unsigned char _cFlag;   /* Flag presence of un-gotten char */
extern unsigned char _cChar;   /* The ungotten char               */

int _cextend = -1;             /* Used for the scan code of an extended key */

/*--------------------------------------------------------------------------*

Name            getch - gets character from console

Usage           int getch(void);

Prototype in    conio.h

Description     getch is a function that reads a single character directly
		from the console, without echoing.

Return value    getch and getche return the character read. There is no
		error return for these two functions.

*---------------------------------------------------------------------------*/

int
ROKEN_LIB_FUNCTION
r_getch(void)
{
	INPUT_RECORD inp;
	DWORD nread;
	HANDLE hin;
	DWORD kbdmode;
	int c, kc;

	/* If a previous r_ungetch() call has been performed, return
	 * the key that it saved.
	 */
	if (_cFlag) {                     /* Previous r_ungetch? */
		_cFlag = 0;                 /* Reset the flag   */
		return (_cChar & 0xff);     /* Get the un-got char  */
		}

	/* If a previous call returned an extended code, return
	 * the scan code for the key.
	 */
	if (_cextend != -1) {
		c = _cextend;
		_cextend = -1;                /* Reset the flag */
		return c;                   /* Return the extended scancode */
		}

	/* Get the keyboard mode, then set it to binary if necessary.
	 * This is done to prevent NT from handling control keys
	 * like Control-C and Control-S.
	 */
	hin = GetStdHandle(STD_INPUT_HANDLE);
	if (GetConsoleMode(hin, &kbdmode) != TRUE)
		return -1;
	if ((kbdmode & KBD_TEXTMODE) != 0)
		if (SetConsoleMode(hin, kbdmode & ~KBD_TEXTMODE) != TRUE)
			return -1;

        /* Get keyboard events until a recognizable one appears.
	 */
	for (;;) {
		if (ReadConsoleInput(hin, &inp, 1, &nread) != TRUE) {
			c = -1;
			break;
			}
		else if ((inp.EventType == KEY_EVENT) &&
			  inp.Event.KeyEvent.bKeyDown) {
		        kc = inp.Event.KeyEvent.wVirtualKeyCode;
			if ((kc == VK_SHIFT) ||
			    (kc == VK_CONTROL) ||
			    (kc == VK_MENU))
				c = -1;	// Ignore!
			else {
//printf("Keydown: Ascii: %d, VK: %d, Scan: %d\n",
//	inp.Event.KeyEvent.uChar.AsciiChar,
//	inp.Event.KeyEvent.wVirtualKeyCode,
//	inp.Event.KeyEvent.wVirtualScanCode);
				c = inp.Event.KeyEvent.uChar.AsciiChar;
				if (c == 0)
					_cextend = kc;
				break;
				}
			}
		}


	/* If we changed the keyboard mode to binary mode earlier,
	 * change it back to its original mode.
	 */
	if ((kbdmode & KBD_TEXTMODE) != 0)
		if (SetConsoleMode(hin, kbdmode) != TRUE)
			return -1;

	if (c == 0x03) {	// Ctrl-C
		ExitProcess(255);
		}
	return c;
}

/*--------------------------------------------------------------------------*

Name            r_ungetch - ungets character to console

Usage           int r_ungetch(int c);

Prototype in    conio.h

Description     r_ungetch pushes the character c back to the console,
                causing c to be the next character read. The r_ungetch
                function fails if it is called more than once before
                the next read.

Return value    r_ungetch returns the character c if it is successful.
                return value of EOF indicates an error.

*---------------------------------------------------------------------------*/

int
ROKEN_LIB_FUNCTION
r_ungetch(int c)
{
	if (_cFlag)
		return(EOF);

	_cFlag = 1;
	return ((_cChar = c) & 0xff);
}
