/*
 * Copyright (c) 2003 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden). 
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 */

#include "tsasl_locl.h"

RCSID("$Id$");

struct tsasl_anonymous {
    char *ta_name;
    int   ta_flags;
#define TA_FLAGS_EMPTY	1
};

int
_tsasl_anonymous_auth(struct tsasl_peer *peer, int flags,
		      struct tsasl_buffer *in,
		      struct tsasl_buffer *out)
{
    struct tsasl_anonymous *ta;
    char *buf;

    ta = peer->tp_mechprivate;
	
    if (in == NULL && out == NULL) {
	if (ta) {
	    if (ta->ta_name)
		free(ta->ta_name);
	    free(ta);
	}
	peer->tp_mechprivate = NULL;
	return 0;
    }

    if (peer->tp_flags & 
	(TSASL_FLAGS_INTEGRITY|TSASL_FLAGS_CONFIDENTIALITY)) {
	_tsasl_set_errorstring(peer,
			       "anon: requested int or conf, "
			       "can't provide that");
	return TSASL_ERROR;
    }

    if (ta == NULL) {
	if ((ta = malloc(sizeof(*ta))) == NULL)
	    return ENOMEM;
	memset(ta, 0, sizeof(*ta));
	ta->ta_flags = 0;
	ta->ta_name = NULL;

	peer->tp_mechprivate = ta;
    }


    if (peer->tp_flags & TSASL_FLAGS_INITIATOR) {
	buf = strdup("anonymous");
	if (buf == NULL) {
	    _tsasl_set_errorstring(peer, "out of mem");
	    return TSASL_ERROR;
	}

	out->tb_data = buf;
	out->tb_size = strlen(buf);

	peer->tp_ctxflags |= TP_CTX_FLAGS_AUTHDONE;
    } else {
	int i;

	if (in->tb_size == 0) {
	    if (ta->ta_flags & TA_FLAGS_EMPTY) {
		_tsasl_set_errorstring(peer, "anon: got empty "
				       "message twice");
		return TSASL_ERROR;
	    }
	    ta->ta_flags |= TA_FLAGS_EMPTY;
	    return TSASL_CONTINUE;
	}


	/* 
	 * Here rfc2248 think that I should include a RFC822
	 * addr-spec parser and the token parser. I'm lazy and
	 * think that a printable-ascii is just fine.
	 */

	buf = in->tb_data;
	for (i = 0; i < in->tb_size; i++) {
	    if (buf[i] < 0x20 || buf[i] > 0x7E)
		break;
	}
	if (i < in->tb_size) {
	    _tsasl_set_errorstring(peer,
				   "anon: trace string contain garbage");
	    return TSASL_ERROR;
			
	}

	asprintf(&buf, "ANONYMOUS:%.*s", 
		 in->tb_size, (char *)in->tb_data);
	if (buf == NULL) {
	    _tsasl_set_errorstring(peer, "out of mem");
	    return TSASL_ERROR;
	}
	tsasl_authentication_id_set(peer, buf);
	free(buf);
			
	peer->tp_ctxflags |= TP_CTX_FLAGS_AUTHDONE;
    }

    return TSASL_DONE;
}

int
_tsasl_anonymous_wrap(struct tsasl_peer *peer, int flags, 
		      struct tsasl_buffer *in,
		      struct tsasl_buffer *out)
{
    if (in->tb_size == 0) {
	out->tb_size = 0;
	out->tb_data = NULL;
	return 0;
    }
    out->tb_data = malloc(in->tb_size);
    if (out->tb_data == NULL) {
	_tsasl_set_errorstring(peer, "anon: out of memory");
	return TSASL_ERROR;
    }
    out->tb_size = in->tb_size;
    memcpy(out->tb_data, in->tb_data, in->tb_size);
    return 0;
}

int
_tsasl_anonymous_status(struct tsasl_peer *peer, char **status)
{
    asprintf(status, "anonymous");
    return 0;
}
