/*
 * Copyright (c) 2003 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden). 
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 */

/* $Id$ */

#include <config.h>

#ifdef __BORLANDC__
#include <krb5-types.h>
#endif

#include <sys/types.h>

#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <base64.h>

#include "tsasl.h"
#include "tsasl-private.h"

struct tsasl_peer {
    int tp_flags;
    int tp_type;
    int tp_ctxflags;
#define TP_CTX_FLAGS_AUTHDONE	1
#define TP_CTX_FLAGS_INT	2
#define TP_CTX_FLAGS_CONF	4
    char *tp_errstring;
    char *tp_realm;
    char *tp_hostname;
    char *tp_service;
    char *tp_servicename;
    char *tp_authorization_id;
    char *tp_authentication_id;
    void *tp_mechprivate;
};

#define TSASL_FLAGS_REQ_MASK	(TSASL_FLAGS_CONFIDENTIALITY | \
				 TSASL_FLAGS_INTEGRITY)
#define TSASL_FLAGS_NEG_REQ_MASK TSASL_FLAGS_ANONYMOUS

typedef enum { TSASL_MD5 = 0,
	       TSASL_GSSAPI = 1,
	       TSASL_ANONYMOUS = 2
} tsasl_type;

typedef int (*tsasl_auth_func)(struct tsasl_peer *, int,
			       struct tsasl_buffer *,
			       struct tsasl_buffer *);
typedef int (*tsasl_wrap_func)(struct tsasl_peer *, int, 
			       struct tsasl_buffer *,
			       struct tsasl_buffer *);

typedef int (*tsasl_status_func)(struct tsasl_peer *, char **);

#define MECH_FUNC(name)							\
int _tsasl_##name##_auth(struct tsasl_peer *, int,			\
		         struct tsasl_buffer *, struct tsasl_buffer *);	\
int _tsasl_##name##_wrap(struct tsasl_peer *,int,			\
			 struct tsasl_buffer *,struct tsasl_buffer *);	\
int _tsasl_##name##_status(struct tsasl_peer *, char **)


MECH_FUNC(md5);
MECH_FUNC(gss);
MECH_FUNC(anonymous);

#undef MECH_FUNC

int	_tsasl_password_get(struct tsasl_peer *, char *, size_t);

#ifdef TSASL_MAIN
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN tsasl_auth_db_func _tsasl_auth_func;
