/*
 * Copyright (c) 1998-2000 Thomas Nystrm and Stacken Computer Club
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define STRICT
#define _WXRTL_DLL
#include "wxrtl.h"
#pragma hdrstop

#include <owl\dialog.h>
#include <owl\radiobut.h>
#include <owl\checkbox.h>
#include <owl\edit.h>
#include <owl\combobox.h>
#include <owl\opensave.h>
#include <string.h>
#include "edititem.h"

#include "wxrtl.rh"

//----------------------------------------------------------------

EditItem::EditItem(TWindow *parent, TListWindow *lw, int index, 
	bool editLabel)
	: TDialog(parent, IDD_EDITITEM, ThisModule)
{
	TStatic *s;
	LV_COLUMN c;

	EditLabel = editLabel;
	Index = index;
	LW = lw;

	s = new TStatic(this, IDC_EILABELHDR, sizeof(trfbuf.labelhdr));
	s->EnableTransfer();
	new TEdit(this, IDC_EILABEL, sizeof(trfbuf.label));

	s = new TStatic(this, IDC_EIVALUEHDR, sizeof(trfbuf.valuehdr));
	s->EnableTransfer();
	new TEdit(this, IDC_EIVALUE, sizeof(trfbuf.value));

	SetTransferBuffer(&trfbuf);

	ZeroMemory(&trfbuf, sizeof(trfbuf));

	c.pszText = trfbuf.labelhdr;
	c.cchTextMax = sizeof(trfbuf.labelhdr);
	c.mask = LVCF_TEXT;
	lw->GetColumn(0, &c);

	c.pszText = trfbuf.valuehdr;
	c.cchTextMax = sizeof(trfbuf.valuehdr);
	c.mask = LVCF_TEXT;
	lw->GetColumn(1, &c);

	TListWindItem item(trfbuf.label, 0, sizeof(trfbuf.label));
	item.SetIndex(Index);
	lw->GetItem(item);

	item.SetText(trfbuf.value, sizeof(trfbuf.value));
	item.SetSubItem(1);
	lw->GetItem(item);	
}

void
EditItem::SetupWindow()
{
	TEdit *e;

	TDialog::SetupWindow();

	if (!EditLabel) {
		e = dynamic_cast<TEdit *>(ChildWithId(IDC_EILABEL));
		if (e)
			e->SetReadOnly(true);
		}
}

bool
EditItem::EvInitDialog(THandle hFocus)
{
	TEdit *e;

	TDialog::EvInitDialog(hFocus);
	
	if (!EditLabel) {	// We decides who shall have focus at startup
		e = dynamic_cast<TEdit *>(ChildWithId(IDC_EIVALUE));
		if (e) {
			SetControlFocus(*e);
			return false;
			}
		}

	return true;
}

void
EditItem::CloseWindow(int code)
{
	if (code == IDOK) {
		TransferData(tdGetData);
		
		if (EditLabel)
			LW->SetItemText(Index, TListWindItem(trfbuf.label));
		LW->SetItemText(Index, TListWindItem(trfbuf.value, 1));
		}
	TDialog::CloseWindow(code);
}

//----------------------------------------------------------------

